@@ -24,11 +24,10 @@ int main()
 	candle.controlMd80Mode(ids[0], mab::Md80Mode_E::VELOCITY_PID);	// Set mode to velocity PID
 	candle.controlMd80Enable(ids[0], true);							// Enable the drive
 
-	// Now we set up Velocity Regulator, and additionaly max output velocity just to be on a safe side
-	candle.md80s[0].setVelocityControllerParams(0.05f, 0.25f, 0.0, 1.0f);
-	candle.md80s[0].setMaxVelocity(30.0f);
+	// Uncomment the line below to change the *.cfg file defaults
+	// candle.md80s[0].setVelocityControllerParams(0.05f, 0.25f, 0.0, 1.0f);
 
-	// To reload default regulator parameters, simply disable the drive (contorlMd80Enable(id, false)),
+	// To reload default controller parameters, simply disable the drive (contorlMd80Enable(id, false)),
 	// stop the communications (candle.end()) or power cycle the drive (off-on).
 
 	float t = 0.0f;
